@@ -21,6 +21,7 @@
 # - Nicolo Magini <nicolo.magini@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Gabriele Fronze' <gfronze@cern.ch>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
@@ -35,10 +36,12 @@ import logging
 import time
 
 from rucio.client.client import Client
-from rucio.common.exception import (RucioException, RSEBlacklisted, DataIdentifierAlreadyExists,
-                                    DataIdentifierNotFound, NoFilesUploaded, NotAllFilesUploaded,
-                                    ResourceTemporaryUnavailable, ServiceUnavailable, InputValidationError)
-from rucio.common.utils import adler32, detect_client_location, execute, generate_uuid, md5, send_trace, GLOBALLY_SUPPORTED_CHECKSUMS
+from rucio.common.config import config_get_int
+from rucio.common.exception import (RucioException, RSEBlacklisted, DataIdentifierAlreadyExists, RSEOperationNotSupported,
+                                    DataIdentifierNotFound, NoFilesUploaded, NotAllFilesUploaded, FileReplicaAlreadyExists,
+                                    ResourceTemporaryUnavailable, ServiceUnavailable, InputValidationError, RSEChecksumUnavailable)
+from rucio.common.utils import (adler32, detect_client_location, execute, generate_uuid, make_valid_did, md5, send_trace,
+                                retry, GLOBALLY_SUPPORTED_CHECKSUMS)
 from rucio.rse import rsemanager as rsemgr
 from rucio import version
 
@@ -57,6 +60,7 @@ class UploadClient:
             logger.disabled = True
 
         self.logger = logger
+
         self.client = _client if _client else Client()
         self.client_location = detect_client_location()
         # if token should be used, use only JWT tokens
@@ -70,6 +74,8 @@ class UploadClient:
         self.trace = {}
         self.trace['hostname'] = socket.getfqdn()
         self.trace['account'] = self.client.account
+        if self.client.vo != 'def':
+            self.trace['vo'] = self.client.vo
         self.trace['eventType'] = 'upload'
         self.trace['eventVersion'] = version.RUCIO_VERSION[0]
 
@@ -100,11 +106,11 @@ class UploadClient:
         :raises NotAllFilesUploaded: if not all files were successfully uploaded
         """
         logger = self.logger
-
         self.trace['uuid'] = generate_uuid()
 
         # check given sources, resolve dirs into files, and collect meta infos
         files = self._collect_and_validate_file_info(items)
+        logger.debug('Num. of files that upload client is processing: {}'.format(len(files)))
 
         # check if RSE of every file is available for writing
         # and cache rse settings
@@ -113,7 +119,7 @@ class UploadClient:
         for file in files:
             rse = file['rse']
             if not self.rses.get(rse):
-                rse_settings = self.rses.setdefault(rse, rsemgr.get_rse_info(rse))
+                rse_settings = self.rses.setdefault(rse, rsemgr.get_rse_info(rse, vo=self.client.vo))
                 if rse_settings['availability_write'] != 1:
                     raise RSEBlacklisted('%s is blacklisted for writing. No actions have been taken' % rse)
 
@@ -128,6 +134,7 @@ class UploadClient:
         wrong_dids = registered_file_dids.intersection(registered_dataset_dids)
         if len(wrong_dids):
             raise InputValidationError('DIDs used to address both files and datasets: %s' % str(wrong_dids))
+        logger.debug('Input validation done.')
 
         # clear this set again to ensure that we only try to register datasets once
         registered_dataset_dids = set()
@@ -177,6 +184,7 @@ class UploadClient:
             if (self.client_location and 'lan' in rse_settings['domain'] and 'site' in rse_attributes):
                 if self.client_location['site'] == rse_attributes['site']:
                     domain = 'lan'
+            logger.debug('{} domain is used for the upload'.format(domain))
 
             if not no_register and not register_after_upload:
                 self._register_file(file, registered_dataset_dids)
@@ -193,16 +201,16 @@ class UploadClient:
                         logger.info('File already exists on RSE. Previous left overs will be overwritten.')
                         delete_existing = True
             elif not is_deterministic and not no_register:
-                if rsemgr.exists(rse_settings, pfn, domain=domain, auth_token=self.auth_token):
+                if rsemgr.exists(rse_settings, pfn, domain=domain, auth_token=self.auth_token, logger=logger):
                     logger.info('File already exists on RSE with given pfn. Skipping upload. Existing replica has to be removed first.')
                     trace['stateReason'] = 'File already exists'
                     continue
-                elif rsemgr.exists(rse_settings, file_did, domain=domain, auth_token=self.auth_token):
+                elif rsemgr.exists(rse_settings, file_did, domain=domain, auth_token=self.auth_token, logger=logger):
                     logger.info('File already exists on RSE with different pfn. Skipping upload.')
                     trace['stateReason'] = 'File already exists'
                     continue
             else:
-                if rsemgr.exists(rse_settings, pfn if pfn else file_did, domain=domain, auth_token=self.auth_token):
+                if rsemgr.exists(rse_settings, pfn if pfn else file_did, domain=domain, auth_token=self.auth_token, logger=logger):
                     logger.info('File already exists on RSE. Skipping upload')
                     trace['stateReason'] = 'File already exists'
                     continue
@@ -212,6 +220,7 @@ class UploadClient:
             protocols.reverse()
             success = False
             state_reason = ''
+            logger.debug(str(protocols))
             while not success and len(protocols):
                 protocol = protocols.pop()
                 cur_scheme = protocol['scheme']
@@ -233,23 +242,22 @@ class UploadClient:
 
                 trace['protocol'] = cur_scheme
                 trace['transferStart'] = time.time()
+                logger.debug('Processing upload with the domain: {}'.format(domain))
                 try:
-                    state = rsemgr.upload(rse_settings=rse_settings,
-                                          lfns=lfn,
-                                          domain=domain,
-                                          source_dir=file['dirname'],
-                                          force_scheme=cur_scheme,
-                                          force_pfn=pfn,
-                                          transfer_timeout=file.get('transfer_timeout'),
-                                          delete_existing=delete_existing,
-                                          sign_service=sign_service,
-                                          auth_token=self.auth_token,
-                                          logger=logger)
-                    success = state['success']
-                    file['upload_result'] = state
-                except (ServiceUnavailable, ResourceTemporaryUnavailable) as error:
+                    pfn = self._upload_item(rse_settings=rse_settings,
+                                            lfn=lfn,
+                                            source_dir=file['dirname'],
+                                            force_scheme=cur_scheme,
+                                            force_pfn=pfn,
+                                            transfer_timeout=file.get('transfer_timeout'),
+                                            delete_existing=delete_existing,
+                                            sign_service=sign_service)
+                    logger.debug('Upload done.')
+                    success = True
+                    file['upload_result'] = {0: True, 1: None, 'success': True, 'pfn': pfn}  # needs to be removed
+                except (ServiceUnavailable, ResourceTemporaryUnavailable, RSEOperationNotSupported, RucioException) as error:
                     logger.warning('Upload attempt failed')
-                    logger.debug('Exception: %s' % str(error))
+                    logger.info('Exception: %s' % str(error))
                     state_reason = str(error)
 
             if success:
@@ -276,7 +284,7 @@ class UploadClient:
                         self.client.attach_dids(file['dataset_scope'], file['dataset_name'], [file_did])
                     except Exception as error:
                         logger.warning('Failed to attach file to the dataset')
-                        logger.debug(error)
+                        logger.debug('Attaching to dataset {}'.format(str(error)))
             else:
                 trace['clientState'] = 'FAILED'
                 trace['stateReason'] = state_reason
@@ -284,6 +292,7 @@ class UploadClient:
                 logger.error('Failed to upload file %s' % basename)
 
         if summary_file_path:
+            logger.debug('Summary will be available at {}'.format(summary_file_path))
             final_summary = {}
             for file in summary:
                 file_scope = file['did_scope']
@@ -300,7 +309,7 @@ class UploadClient:
                     if checksum_name in file:
                         final_summary[file_did_str][checksum_name] = file[checksum_name]
 
-            with open(summary_file_path, 'wb') as summary_file:
+            with open(summary_file_path, 'w') as summary_file:
                 json.dump(final_summary, summary_file, sort_keys=True, indent=1)
 
         if num_succeeded == 0:
@@ -323,6 +332,12 @@ class UploadClient:
         """
         logger = self.logger
         logger.debug('Registering file')
+
+        # verification whether the scope exists
+        account_scopes = self.client.list_scopes_for_account(self.client.account)
+        if file['did_scope'] not in account_scopes:
+            logger.warning('Scope {} not found for the account {}.'.format(file['did_scope'], self.client.account))
+
         rse = file['rse']
         dataset_did_str = file.get('dataset_did_str')
         # register a dataset if we need to
@@ -495,6 +510,163 @@ class UploadClient:
             replica['pfn'] = pfn
         return replica
 
+    def _upload_item(self, rse_settings, lfn, source_dir=None, force_pfn=None, force_scheme=None, transfer_timeout=None, delete_existing=False, sign_service=None):
+        """
+            Uploads a file to the connected storage.
+
+            :param lfn:         a single dict containing 'scope' and 'name'.
+                                Example:
+                             {'name': '1_rse_local_put.raw', 'scope': 'user.jdoe', 'filesize': 42, 'adler32': '87HS3J968JSNWID'}
+                              If the 'filename' key is present, it will be used by Rucio as the actual name of the file on disk (separate from the Rucio 'name').
+            :param source_dir:  path to the local directory including the source files
+            :param force_pfn: use the given PFN -- can lead to dark data, use sparingly
+            :param force_scheme: use the given protocol scheme, overriding the protocol priority in the RSE description
+            :param transfer_timeout: set this timeout (in seconds) for the transfers, for protocols that support it
+            :param sign_service: use the given service (e.g. gcs, s3, swift) to sign the URL
+
+            :raises RucioException(msg): general exception with msg for more details.
+        """
+        logger = self.logger
+
+        # Construct protocol for write and read operation.
+        protocol_write = self._create_protocol(rse_settings, 'write', force_scheme=force_scheme)
+        protocol_read = self._create_protocol(rse_settings, 'read')
+
+        base_name = lfn.get('filename', lfn['name'])
+        name = lfn.get('name', base_name)
+        scope = lfn['scope']
+
+        # Conditional lfn properties
+        if 'adler32' not in lfn:
+            logger.warning('Missing checksum for file %s:%s' % (lfn['scope'], name))
+
+        # Getting pfn
+        pfn = None
+        readpfn = None
+        try:
+            pfn = list(protocol_write.lfns2pfns(make_valid_did(lfn)).values())[0]
+            readpfn = list(protocol_read.lfns2pfns(make_valid_did(lfn)).values())[0]
+            logger.debug('The PFN created from the LFN: {}'.format(pfn))
+        except Exception as error:
+            logger.warning('Failed to create PFN for LFN: %s' % lfn)
+            logger.debug(str(error))
+        if force_pfn:
+            pfn = force_pfn
+            readpfn = pfn
+            logger.debug('The given PFN is used: {}'.format(pfn))
+
+        # Auth. mostly for object stores
+        if sign_service:
+            pfn = self.client.get_signed_url(rse_settings['rse'], sign_service, 'write', pfn)       # NOQA pylint: disable=undefined-variable
+            readpfn = self.client.get_signed_url(rse_settings['rse'], sign_service, 'read', pfn)    # NOQA pylint: disable=undefined-variable
+
+        # Create a name of tmp file if renaming operation is supported
+        pfn_tmp = '%s.rucio.upload' % pfn if protocol_write.renaming else pfn
+        readpfn_tmp = '%s.rucio.upload' % readpfn if protocol_write.renaming else readpfn
+
+        # Either DID eixsts or not register_after_upload
+        if protocol_write.overwrite is False and delete_existing is False and protocol_read.exists(readpfn):
+            raise FileReplicaAlreadyExists('File %s in scope %s already exists on storage as PFN %s' % (name, scope, pfn))  # wrong exception ?
+
+        # Removing tmp from earlier attempts
+        if protocol_read.exists('%s.rucio.upload' % readpfn):
+            logger.debug('Removing remains of previous upload attemtps.')
+            try:
+                # Construct protocol for delete operation.
+                protocol_delete = self._create_protocol(rse_settings, 'delete')
+                protocol_delete.delete('%s.rucio.upload' % list(protocol_delete.lfns2pfns(make_valid_did(lfn)).values())[0])
+                protocol_delete.close()
+            except Exception as e:
+                raise RSEOperationNotSupported('Unable to remove temporary file %s.rucio.upload: %s' % (pfn, str(e)))
+
+        # Removing not registered files from earlier attempts
+        if delete_existing:
+            logger.debug('Removing not-registered remains of previous upload attemtps.')
+            try:
+                # Construct protocol for delete operation.
+                protocol_delete = self._create_protocol(rse_settings, 'delete')
+                protocol_delete.delete('%s' % list(protocol_delete.lfns2pfns(make_valid_did(lfn)).values())[0])
+                protocol_delete.close()
+            except Exception as error:
+                raise RSEOperationNotSupported('Unable to remove file %s: %s' % (pfn, str(error)))
+
+        # Process the upload of the tmp file
+        try:
+            retry(protocol_write.put, base_name, pfn_tmp, source_dir, transfer_timeout=transfer_timeout)(mtries=2, logger=logger)
+            logger.info('Successful upload of temporary file. {}'.format(pfn_tmp))
+        except Exception as error:
+            raise RSEOperationNotSupported(str(error))
+
+        # Is stat after that upload allowed?
+        skip_upload_stat = rse_settings.get('skip_upload_stat', False)
+
+        # Checksum verification, obsolete, see Gabriele changes.
+        if not skip_upload_stat:
+            try:
+                stats = self._retry_protocol_stat(protocol_read, readpfn_tmp)
+                if not isinstance(stats, dict):
+                    raise RucioException('Could not get protocol.stats for given PFN: %s' % pfn)
+
+                # The checksum and filesize check
+                if ('filesize' in stats) and ('filesize' in lfn):
+                    if int(stats['filesize']) != int(lfn['filesize']):
+                        raise RucioException('Filesize mismatch. Source: %s Destination: %s' % (lfn['filesize'], stats['filesize']))
+                if rse_settings['verify_checksum'] is not False:
+                    if ('adler32' in stats) and ('adler32' in lfn):
+                        if stats['adler32'] != lfn['adler32']:
+                            raise RucioException('Checksum mismatch. Source: %s Destination: %s' % (lfn['adler32'], stats['adler32']))
+
+            except Exception as error:
+                raise error
+
+        # The upload finished successful and the file can be renamed
+        try:
+            if protocol_write.renaming:
+                logger.debug('Renaming file %s to %s' % (pfn_tmp, pfn))
+                protocol_write.rename(pfn_tmp, pfn)
+        except Exception as e:
+            raise RucioException('Unable to rename the tmp file %s.' % pfn_tmp)
+
+        protocol_write.close()
+        protocol_read.close()
+
+        return pfn
+
+    def _retry_protocol_stat(self, protocol, pfn):
+        """
+        try to stat file, on fail try again 1s, 2s, 4s, 8s, 16s, 32s later. Fail is all fail
+        :param protocol     The protocol to use to reach this file
+        :param pfn          Physical file name of the target for the protocol stat
+        """
+        retries = config_get_int('client', 'protocol_stat_retries', raise_exception=False, default=6)
+        for attempt in range(retries):
+            try:
+                stats = protocol.stat(pfn)
+                return stats
+            except RSEChecksumUnavailable as error:
+                # The stat succeeded here, but the checksum failed
+                raise error
+            except Exception as error:
+                time.sleep(2**attempt)
+        return protocol.stat(pfn)
+
+    def _create_protocol(self, rse_settings, operation, force_scheme=None):
+        """
+        Protol construction.
+        :param: rse_settings        rse_settings
+        :param: operation           activity, e.g. read, write, delete etc.
+        :param: force_scheme        custom scheme
+        :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
+        """
+        try:
+            protocol = rsemgr.create_protocol(rse_settings, operation, scheme=force_scheme, auth_token=self.auth_token)
+            protocol.connect()
+        except Exception as error:
+            self.logger.warning('Failed to create protocol for operation: %s' % operation)
+            self.logger.debug('scheme: %s, exception: %s' % (force_scheme, error))
+            raise error
+        return protocol
+
     def _send_trace(self, trace):
         """
         Checks if sending trace is allowed and send the trace.
