@@ -20,6 +20,8 @@
 # - Martin Barisits <martin.barisits@cern.ch>, 2014
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2017
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 
 from __future__ import print_function
 
@@ -29,7 +31,7 @@ from nose.tools import raises
 
 from rucio.client.baseclient import BaseClient
 from rucio.client.client import Client
-from rucio.common.config import config_get
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.utils import get_tmp_dir
 from rucio.common.exception import CannotAuthenticate, ClientProtocolNotSupported
 
@@ -41,6 +43,17 @@ class TestBaseClient(object):
         '''
         __init__
         '''
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            try:
+                remove(get_tmp_dir() + '/.rucio_root@%s/auth_token_root' % self.vo['vo'])
+            except OSError as error:
+                if error.args[0] != 2:
+                    raise error
+
+        else:
+            self.vo = {}
+
         self.cacert = config_get('test', 'cacert')
         self.usercert = config_get('test', 'usercert')
         self.userkey = config_get('test', 'userkey')
@@ -53,37 +66,37 @@ class TestBaseClient(object):
     def testUserpass(self):
         """ CLIENTS (BASECLIENT): authenticate with userpass."""
         creds = {'username': 'ddmlab', 'password': 'secret'}
-        BaseClient(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds)
+        BaseClient(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds, **self.vo)
 
     @raises(CannotAuthenticate)
     def testUserpassWrongCreds(self):
         """ CLIENTS (BASECLIENT): try to authenticate with wrong username."""
         creds = {'username': 'wrong', 'password': 'secret'}
-        BaseClient(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds)
+        BaseClient(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds, **self.vo)
 
     @raises(CannotAuthenticate)
     def testUserpassNoCACert(self):
         """ CLIENTS (BASECLIENT): authenticate with userpass without ca cert."""
         creds = {'username': 'wrong', 'password': 'secret'}
-        BaseClient(account='root', auth_type='userpass', creds=creds)
+        BaseClient(account='root', auth_type='userpass', creds=creds, **self.vo)
 
     def testx509(self):
         """ CLIENTS (BASECLIENT): authenticate with x509."""
         creds = {'client_cert': self.usercert,
                  'client_key': self.userkey}
-        BaseClient(account='root', ca_cert=self.cacert, auth_type='x509', creds=creds)
+        BaseClient(account='root', ca_cert=self.cacert, auth_type='x509', creds=creds, **self.vo)
 
     @raises(CannotAuthenticate)
     def testx509NonExistingCert(self):
         """ CLIENTS (BASECLIENT): authenticate with x509 with missing certificate."""
         creds = {'client_cert': '/opt/rucio/etc/web/notthere.crt'}
-        BaseClient(account='root', ca_cert=self.cacert, auth_type='x509', creds=creds)
+        BaseClient(account='root', ca_cert=self.cacert, auth_type='x509', creds=creds, **self.vo)
 
     @raises(ClientProtocolNotSupported)
     def testClientProtocolNotSupported(self):
         """ CLIENTS (BASECLIENT): try to pass an host with a not supported protocol."""
         creds = {'username': 'ddmlab', 'password': 'secret'}
-        BaseClient(rucio_host='localhost', auth_host='junk://localhost', account='root', auth_type='userpass', creds=creds)
+        BaseClient(rucio_host='localhost', auth_host='junk://localhost', account='root', auth_type='userpass', creds=creds, **self.vo)
 
 
 class TestRucioClients(object):
@@ -93,6 +106,11 @@ class TestRucioClients(object):
         '''
         setup
         '''
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
+
         self.cacert = config_get('test', 'cacert')
         self.marker = '$> '
 
@@ -100,6 +118,6 @@ class TestRucioClients(object):
         """ PING (CLIENT): Ping Rucio """
         creds = {'username': 'ddmlab', 'password': 'secret'}
 
-        client = Client(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds)
+        client = Client(account='root', ca_cert=self.cacert, auth_type='userpass', creds=creds, **self.vo)
 
         print(client.ping())
