@@ -11,10 +11,12 @@
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Cedric Serfon,<cedric.serfon@cern.ch>, 2019
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 
 from nose.tools import assert_equal, assert_in
 
 from rucio.db.sqla import session, models
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalAccount
 from rucio.core import account_counter, rse_counter
 from rucio.core.account import get_usage
@@ -24,10 +26,15 @@ from rucio.daemons.abacus.account import account_update
 
 
 class TestCoreRSECounter():
+    def setup(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
 
     def test_inc_dec_get_counter(self):
         """ RSE COUNTER (CORE): Increase, decrease and get counter """
-        rse_id = get_rse_id(rse='MOCK')
+        rse_id = get_rse_id(rse='MOCK', **self.vo)
         rse_update(once=True)
         rse_counter.del_counter(rse_id=rse_id)
         rse_counter.add_counter(rse_id=rse_id)
@@ -85,12 +92,17 @@ class TestCoreRSECounter():
 
 
 class TestCoreAccountCounter():
+    def setup(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
 
     def test_inc_dec_get_counter(self):
         """ACCOUNT COUNTER (CORE): Increase, decrease and get counter """
         account_update(once=True)
-        rse_id = get_rse_id(rse='MOCK')
-        account = InternalAccount('jdoe')
+        rse_id = get_rse_id(rse='MOCK', **self.vo)
+        account = InternalAccount('jdoe', **self.vo)
         account_counter.del_counter(rse_id=rse_id, account=account)
         account_counter.add_counter(rse_id=rse_id, account=account)
         cnt = get_usage(rse_id=rse_id, account=account)
