@@ -9,12 +9,14 @@
 # - Martin Barisits, <martin.barisits@cern.ch>, 2013-2017
 # - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
 # - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
 
 from random import choice
 from string import ascii_uppercase, digits, ascii_lowercase
 
 from nose.tools import assert_equal, raises, assert_raises
 
+from rucio.common.config import config_get, config_get_bool
 from rucio.core import rse
 from rucio.core import rse_expression_parser
 from rucio.client.rseclient import RSEClient
@@ -36,17 +38,24 @@ def attribute_name_generator(size=10):
 class TestRSEExpressionParserCore(object):
 
     def __init__(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            self.filter = {'filter': self.vo}
+        else:
+            self.vo = {}
+            self.filter = {'filter': {'vo': 'def'}}
+
         self.rse1 = rse_name_generator()
         self.rse2 = rse_name_generator()
         self.rse3 = rse_name_generator()
         self.rse4 = rse_name_generator()
         self.rse5 = rse_name_generator()
 
-        self.rse1_id = rse.add_rse(self.rse1)
-        self.rse2_id = rse.add_rse(self.rse2)
-        self.rse3_id = rse.add_rse(self.rse3)
-        self.rse4_id = rse.add_rse(self.rse4)
-        self.rse5_id = rse.add_rse(self.rse5)
+        self.rse1_id = rse.add_rse(self.rse1, **self.vo)
+        self.rse2_id = rse.add_rse(self.rse2, **self.vo)
+        self.rse3_id = rse.add_rse(self.rse3, **self.vo)
+        self.rse4_id = rse.add_rse(self.rse4, **self.vo)
+        self.rse5_id = rse.add_rse(self.rse5, **self.vo)
 
         # Add Attributes
         self.attribute = attribute_name_generator()
@@ -78,77 +87,76 @@ class TestRSEExpressionParserCore(object):
     @raises(InvalidRSEExpression)
     def test_unconnected_operator(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test invalid rse expression: unconnected operator"""
-        rse_expression_parser.parse_expression("TEST_RSE1|")
+        rse_expression_parser.parse_expression("TEST_RSE1|", **self.filter)
 
     @raises(InvalidRSEExpression)
     def test_wrong_parantheses(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test invalid rse expression: wrong parantheses """
-        rse_expression_parser.parse_expression("TEST_RSE1)")
+        rse_expression_parser.parse_expression("TEST_RSE1)", **self.filter)
 
     @raises(InvalidRSEExpression)
     def test_unknown_rse(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test unknown RSE """
-        rse_expression_parser.parse_expression("TEST_RSE999")
+        rse_expression_parser.parse_expression("TEST_RSE999", **self.filter)
 
     def test_simple_rse_reference(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test simple RSE reference """
-        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression(self.rse1)], [self.rse1_id])
+        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression(self.rse1, **self.filter)], [self.rse1_id])
 
     def test_attribute_reference(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test simple RSE attribute reference """
-        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s=uk" % self.attribute)], [self.rse4_id])
+        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s=uk" % self.attribute, **self.filter)], [self.rse4_id])
 
     def test_all_rse(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test reference on all RSE """
-        all_rses = rse.list_rses()
-        assert_equal(sorted(rse_expression_parser.parse_expression("*"), key=lambda rse: rse['rse']), sorted(all_rses, key=lambda rse: rse['rse']))
+        all_rses = rse.list_rses(filters=self.filter['filter'])
+        assert_equal(sorted(rse_expression_parser.parse_expression("*", **self.filter), key=lambda rse: rse['rse']), sorted(all_rses, key=lambda rse: rse['rse']))
 
     def test_tag_reference(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test simple RSE tag reference """
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression(self.tag1)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression(self.tag1, **self.filter)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
 
     def test_parantheses(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test parantheses """
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(%s)" % self.tag1)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(%s)" % self.tag1, **self.filter)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
 
     def test_union(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test union operator """
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s|%s" % (self.tag1, self.tag2))]), sorted([self.rse1_id, self.rse2_id, self.rse3_id, self.rse4_id, self.rse5_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s|%s" % (self.tag1, self.tag2), **self.filter)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id, self.rse4_id, self.rse5_id]))
 
     def test_complement(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test complement operator """
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\%s" % (self.tag1, self.rse3))]), sorted([self.rse1_id, self.rse2_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\%s" % (self.tag1, self.rse3), **self.filter)]), sorted([self.rse1_id, self.rse2_id]))
 
     def test_intersect(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test intersect operator """
-        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s&%s=uk" % (self.tag2, self.attribute))], [self.rse4_id])
+        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s&%s=uk" % (self.tag2, self.attribute), **self.filter)], [self.rse4_id])
 
     def test_order_of_operations(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test order of operations """
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\%s|%s=fr" % (self.tag1, self.rse3, self.attribute))]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\(%s|%s=fr)" % (self.tag1, self.rse3, self.attribute))]), sorted([self.rse1_id, self.rse2_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\%s|%s=fr" % (self.tag1, self.rse3, self.attribute), **self.filter)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s\\(%s|%s=fr)" % (self.tag1, self.rse3, self.attribute), **self.filter)]), sorted([self.rse1_id, self.rse2_id]))
 
     def test_complicated_expression_1(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test some complicated expression 1"""
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(%s|%s)\\%s|%s&%s" % (self.tag1, self.tag2, self.tag2, self.tag2, self.tag1))]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(%s|%s)\\%s|%s&%s" % (self.tag1, self.tag2, self.tag2, self.tag2, self.tag1), **self.filter)]), sorted([self.rse1_id, self.rse2_id, self.rse3_id]))
 
     def test_complicated_expression_2(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test some complicated expression 2"""
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(((((%s))))|%s=us)&%s|(%s=at|%s=de)" %
-                                                                                             (self.tag1, self.attribute, self.tag2, self.attribute, self.attribute))]), sorted([self.rse1_id, self.rse2_id, self.rse5_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(((((%s))))|%s=us)&%s|(%s=at|%s=de)" % (self.tag1, self.attribute, self.tag2, self.attribute, self.attribute), **self.filter)]),
+                     sorted([self.rse1_id, self.rse2_id, self.rse5_id]))
 
     def test_complicated_expression_3(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test some complicated expression 3"""
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(*)&%s=at" % self.attribute)]), sorted([self.rse1_id]))
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("(*)&%s=at" % self.attribute, **self.filter)]), sorted([self.rse1_id]))
 
-    @staticmethod
-    def test_list_on_availability():
+    def test_list_on_availability(self):
         """ RSE_EXPRESSION_PARSER (CORE) List rses based on availability filter"""
         rsewrite_name = rse_name_generator()
         rsenowrite_name = rse_name_generator()
 
-        rsewrite_id = rse.add_rse(rsewrite_name)
-        rsenowrite_id = rse.add_rse(rsenowrite_name)
+        rsewrite_id = rse.add_rse(rsewrite_name, **self.vo)
+        rsenowrite_id = rse.add_rse(rsenowrite_name, **self.vo)
 
         attribute = attribute_name_generator()
 
@@ -158,38 +166,46 @@ class TestRSEExpressionParserCore(object):
         rse.update_rse(rsewrite_id, {'availability_write': True})
         rse.update_rse(rsenowrite_id, {'availability_write': False})
 
-        assert_equal(sorted([item['id'] for item in rse_expression_parser.parse_expression("%s=de" % attribute)]),
+        assert_equal(sorted([item['id'] for item in rse_expression_parser.parse_expression("%s=de" % attribute, **self.filter)]),
                      sorted([rsewrite_id, rsenowrite_id]))
 
-        assert_equal(sorted([item['id'] for item in rse_expression_parser.parse_expression("%s=de" % attribute, {'availability_write': True})]),
+        filters = self.filter
+        filters['availability_write'] = True
+        assert_equal(sorted([item['id'] for item in rse_expression_parser.parse_expression("%s=de" % attribute, filters)]),
                      sorted([rsewrite_id]))
 
-        assert_raises(RSEBlacklisted, rse_expression_parser.parse_expression, "%s=de" % attribute, {'availability_write': False})
+        filters['availability_write'] = False
+        assert_raises(RSEBlacklisted, rse_expression_parser.parse_expression, "%s=de" % attribute, filters)
 
     def test_numeric_operators(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test RSE attributes with numeric operations """
-        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s<11" % self.attribute_numeric)], [self.rse1_id])
-        assert_raises(InvalidRSEExpression, rse_expression_parser.parse_expression, "%s<9" % self.attribute_numeric)
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s<21" % self.attribute_numeric)]), sorted([self.rse1_id, self.rse2_id]))
-        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s>49" % self.attribute_numeric)], [self.rse5_id])
-        assert_raises(InvalidRSEExpression, rse_expression_parser.parse_expression, "%s>51" % self.attribute_numeric)
-        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s>30" % self.attribute_numeric)]), sorted([self.rse4_id, self.rse5_id]))
+        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s<11" % self.attribute_numeric, **self.filter)], [self.rse1_id])
+        assert_raises(InvalidRSEExpression, rse_expression_parser.parse_expression, "%s<9" % self.attribute_numeric, **self.filter)
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s<21" % self.attribute_numeric, **self.filter)]), sorted([self.rse1_id, self.rse2_id]))
+        assert_equal([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s>49" % self.attribute_numeric, **self.filter)], [self.rse5_id])
+        assert_raises(InvalidRSEExpression, rse_expression_parser.parse_expression, "%s>51" % self.attribute_numeric, **self.filter)
+        assert_equal(sorted([t_rse['id'] for t_rse in rse_expression_parser.parse_expression("%s>30" % self.attribute_numeric, **self.filter)]), sorted([self.rse4_id, self.rse5_id]))
 
 
 class TestRSEExpressionParserClient(object):
 
     def __init__(self):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            self.vo = {}
+
         self.rse1 = rse_name_generator()
         self.rse2 = rse_name_generator()
         self.rse3 = rse_name_generator()
         self.rse4 = rse_name_generator()
         self.rse5 = rse_name_generator()
 
-        self.rse1_id = rse.add_rse(self.rse1)
-        self.rse2_id = rse.add_rse(self.rse2)
-        self.rse3_id = rse.add_rse(self.rse3)
-        self.rse4_id = rse.add_rse(self.rse4)
-        self.rse5_id = rse.add_rse(self.rse5)
+        self.rse1_id = rse.add_rse(self.rse1, **self.vo)
+        self.rse2_id = rse.add_rse(self.rse2, **self.vo)
+        self.rse3_id = rse.add_rse(self.rse3, **self.vo)
+        self.rse4_id = rse.add_rse(self.rse4, **self.vo)
+        self.rse5_id = rse.add_rse(self.rse5, **self.vo)
 
         # Add Attributes
         self.attribute = attribute_name_generator()
